<?php

/**
 * @file
 * Post update functions for access policy.
 */

/**
 * Add support for revisions.
 */
function access_policy_post_update_revision_support(&$sandbox) {
  $connection = \Drupal::database();
  $entityTypeManager = \Drupal::service('entity_type.manager');
  $storage = \Drupal::service('access_policy.storage.base_field');
  $oldStorage = \Drupal::service('access_policy.storage.entity');
  $entity_types = $entityTypeManager->getDefinitions();

  // Initialize the batch for each entity type.
  // Because we're doing raw mysql updates they are much faster and lighter
  // than going through the entity api. Batching per entity type should be
  // sufficient.
  if (!isset($sandbox['total'])) {

    $supported_entity_types = [];
    foreach ($entity_types as $entity_type) {
      if (\Drupal::service('access_policy.information')->isSupportedEntityType($entity_type) && !$oldStorage->isEmpty($entity_type->id())) {
        $supported_entity_types[] = $entity_type;
      }
    }

    $sandbox['total'] = count($supported_entity_types);
    $sandbox['current'] = 0;
    $sandbox['entity_types'] = $supported_entity_types;
  }

  $current = $sandbox['current'];
  $entity_type = $sandbox['entity_types'][$current];

  // First update the deltas of the original table.
  $table = $oldStorage->getTable($entity_type->id());
  $select = $connection->select($table, 'cap');
  $select->addField('cap', 'entity_id');
  $select->addField('cap', 'access_policy');
  $results = $select->execute();

  $data = $results->fetchAll(\PDO::FETCH_ASSOC);

  if (!empty($data)) {
    // Reset the deltas.
    $deltas = [];
    foreach ($data as $index => $row) {
      $entity_id = $row['entity_id'];
      if (!isset($deltas[$entity_id])) {
        $deltas[$entity_id] = 0;
      }
      $data[$index]['delta'] = (int) $deltas[$entity_id];
      $deltas[$entity_id]++;
    }

    foreach ($data as $row) {
      $query = $connection->update($table)
        ->fields([
          'delta' => $row['delta'],
        ])
        ->condition('entity_id', $row['entity_id'])
        ->condition('access_policy', $row['access_policy']);
      $query->execute();
    }
  }

  // Only copy the content over if the original storage is not empty and the
  // new storage is empty.
  if ($storage->isEmpty($entity_type->id())) {
    $base_table = $entity_type->getBaseTable();
    $bundle_key = $entity_type->getKey('bundle');
    $id_key = $entity_type->getKey('id');
    $revision_key = $entity_type->getKey('revision');
    $langcode_key = $entity_type->getKey('langcode');

    // Do a bulk insert of values into the new table.
    $query = $connection->select($oldStorage->getTable($entity_type->id()), 'cap');

    $query->addExpression("[et].[{$bundle_key}]", 'bundle');
    $query->addExpression(0, 'deleted');
    $query->addExpression('[cap].[entity_id]', 'entity_id');
    $query->addExpression("[et].[{$revision_key}]", 'revision_id');
    $query->addExpression("[et].[{$langcode_key}]", 'langcode');
    $query->addExpression('[cap].delta', 'delta');
    $query->addExpression('[cap].[access_policy]', 'access_policy_target_id');

    $query->condition('cap.access_policy', NULL, 'IS NOT NULL');
    $query->join($base_table, 'et', "et.{$id_key} = cap.entity_id");

    // Insert into data table.
    $connection->insert($storage->getTable($entity_type->id()))
      ->from($query)
      ->execute();

    // Insert into revisions table.
    $connection->insert($entity_type->id() . '_revision__access_policy')
      ->from($query)
      ->execute();
  }

  $sandbox['current']++;
  \Drupal::messenger()->addStatus('Updated ' . $entity_type->id());

  if ($sandbox['current'] >= $sandbox['total']) {
    $sandbox['#finished'] = 1;
  }
  else {
    $sandbox['#finished'] = ($sandbox['current'] / $sandbox['total']);
  }

}
