<?php

namespace Drupal\access_policy;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Class AccessPolicyTypePluginManager.
 *
 * These classes are responsible for access policies get evaluated.
 */
class AccessPolicyTypePluginManager extends DefaultPluginManager {

  /**
   * AccessPolicyTypePluginManager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/AccessPolicyType', $namespaces, $module_handler, 'Drupal\access_policy\Plugin\AccessPolicyType\AccessPolicyTypeInterface', 'Drupal\access_policy\Annotation\AccessPolicyType');

    $this->alterInfo('access_policy_type');
    $this->setCacheBackend($cache_backend, 'access_policy_type');
  }

  /**
   * {@inheritdoc}
   */
  public function getDefinitions() {
    // Fetch and sort definitions by weight.
    $definitions = $this->getCachedDefinitions();
    if (empty($definitions)) {
      $definitions = $this->findDefinitions();
      uasort($definitions, '\Drupal\Component\Utility\SortArray::sortByWeightElement');

      $this->setCachedDefinitions($definitions);
    }

    return $definitions;
  }

}
