<?php

namespace Drupal\access_policy;

use Drupal\Core\Entity\EntityInterface;
use Drupal\access_policy\Entity\AccessPolicyInterface;

/**
 * Service for handling Access Policy database content.
 *
 * This makes it possible to swap between storage engines for access policies
 * assigned to entities.
 *
 * @deprecated in access_policy:1.0.0-beta1 and is removed from access_policy:1.0.0-beta2.
 *   There is no replacement.
 *
 * @package Drupal\access_policy
 */
interface EntityAccessPolicyStorageInterface {

  /**
   * Determine whether this access policy is being used.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy.
   *
   * @return bool
   *   TRUE if it is being used; FALSE otherwise.
   */
  public function isUsed(AccessPolicyInterface $access_policy);

  /**
   * Get the entities that are using this access policy.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy.
   *
   * @return array
   *   Array of entity ids that are using this access policy.
   */
  public function getUsage(AccessPolicyInterface $access_policy);

  /**
   * Get the access policy on an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return string
   *   The current access policy id assigned to an entity.
   */
  public function read(EntityInterface $entity);

  /**
   * Determine whether any record exists in the database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return bool
   *   TRUE if no record exists; FALSE otherwise.
   */
  public function isNull(EntityInterface $entity);

  /**
   * Write the content access policy to the database.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   * @param mixed $access_policy
   *   The access policy id.
   *
   * @throws \Exception
   */
  public function write(EntityInterface $entity, $access_policy);

  /**
   * Delete the access policies assigned to an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  public function delete(EntityInterface $entity);

  /**
   * Get the table name for this entity type.
   *
   * @param string $entity_type
   *   The entity type id.
   *
   * @return string
   *   The table name.
   */
  public function getTable($entity_type);

  /**
   * Get the column name based on a key.
   *
   * @param string $key
   *   The key.
   *
   * @return false|string
   *   The column name.
   */
  public function getColumnName($key);

  /**
   * Determine whether this table is empty.
   *
   * @param string $entity_type
   *   The entity type id.
   *
   * @return bool
   *   TRUE if the table is empty; FALSE otherwise.
   */
  public function isEmpty($entity_type);

}
