<?php

namespace Drupal\access_policy\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormStateInterface;
use Drupal\access_policy\Entity\AccessPolicyInterface;

/**
 * The form for deleting handlers associated with an access policy.
 *
 * @internal
 */
class AccessPolicyRemoveRuleForm extends AccessRuleFormBase {

  /**
   * The handler id.
   *
   * @var string
   */
  protected $id;

  /**
   * The handler type.
   *
   * @var string
   */
  protected $handlerType;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_policy_remove_rule_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, AccessPolicyInterface $access_policy = NULL, $type = NULL, $id = NULL) {
    $this->accessPolicy = $access_policy;
    $this->handlerType = $type;
    $this->id = $id;

    $form = parent::buildForm($form, $form_state);

    $form['message'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Are you sure you want to remove this rule?'),
    ];

    $form['actions']['submit']['#value'] = $this->t('Remove');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->accessPolicy->removeHandler($this->handlerType, $this->id);
    $this->accessPolicy->save();
  }

  /**
   * Checks that the rule is not required and can be deleted.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy entity.
   * @param string $type
   *   The handler type.
   * @param string $id
   *   The handler unique id.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccessPolicyInterface $access_policy = NULL, $type = NULL, $id = NULL) {
    $plugins = $access_policy->getHandlers($type);
    if (isset($plugins[$id])) {
      $plugin = $plugins[$id];
    }

    if (!empty($plugin['required'])) {
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

}
