<?php

namespace Drupal\access_policy\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'content_access_policy' formatter.
 *
 * @FieldFormatter(
 *   id = "content_access_policy",
 *   label = @Translation("Access policy"),
 *   field_types = {
 *     "entity_reference",
 *   }
 * )
 */
class ContentAccessPolicyFormatter extends FormatterBase {

  /**
   * Access policy storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $accessPolicyStorage;

  /**
   * Create an instance of ContentAccessPolicyFormatter.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->accessPolicyStorage = $entity_type_manager->getStorage('access_policy');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      if (!empty($item->value)) {
        $elements[$delta] = [
          '#markup' => $this->accessPolicyStorage->load($item->value)->label(),
        ];
      }
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return $field_definition->getName() === 'access_policy';
  }

}
