<?php

namespace Drupal\access_policy\Plugin\SelectionRule;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Control selection based on bundle.
 *
 * @SelectionRule(
 *   id = "bundle",
 * )
 */
class Bundle extends FieldList {

  /**
   * {@inheritdoc}
   */
  public function isApplicable(EntityInterface $entity) {
    // We can always assume that the entity type has the bundle field.
    // Instead, we validate so that it shows/hides it from the select
    // list on the entity access page.
    return $this->validate($entity, \Drupal::currentUser());
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    return [
      'value' => [],
      "operator" => 'or',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function operators() {
    $operators = [
      'or' => [
        'title' => $this->t('Is one of'),
        'method' => 'validateSimple',
      ],
      'not' => [
        'title' => $this->t('Is none of'),
        'method' => 'validateSimple',
      ],
    ];

    return $operators;
  }

  /**
   * {@inheritdoc}
   */
  protected function valueForm(array &$form, FormStateInterface $form_state) {
    $form['value'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Value'),
      '#options' => $this->getAllowedValues(),
      '#default_value' => $this->settings['value'],
      '#multiple' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildSettingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildSettingsForm($form, $form_state);
    unset($form['field_access']);
    return $form;
  }

  /**
   * Get allowed field value options.
   *
   * @return array
   *   Array of allowed values.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getAllowedValues() {
    $bundleInfo = \Drupal::service('entity_type.bundle.info');
    $bundle_info = $bundleInfo->getBundleInfo($this->definition->getEntityType());

    $bundles = [];
    foreach ($bundle_info as $name => $info) {
      $bundles[$name] = $info['label'];
    }
    return $bundles;
  }

}
